unit f_main;

{$WARN SYMBOL_PLATFORM OFF}

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms, ComObj,
  Dialogs, StdCtrls, Menus, u_obimp_const, u_obimp_codes, u_events, obimp_ax_info, ObimpAX_TLB;

type
  TfrmMain = class(TForm)
    EditAcc: TEdit;
    EditPass: TEdit;
    Label1: TLabel;
    Label2: TLabel;
    ButtonLogin: TButton;
    ButtonLogoff: TButton;
    Memo1: TMemo;
    EditServer: TEdit;
    Label3: TLabel;
    EditPort: TEdit;
    Label4: TLabel;
    ButtonStatus: TButton;
    Label5: TLabel;
    PopupMenuStatus: TPopupMenu;
    MenuItemFFC: TMenuItem;
    MenuItemAtHome: TMenuItem;
    MenuItemAtWork: TMenuItem;
    MenuItemLine1: TMenuItem;
    MenuItemLunch: TMenuItem;
    MenuItemAway: TMenuItem;
    MenuItemNA: TMenuItem;
    MenuItemOccupied: TMenuItem;
    MenuItemDND: TMenuItem;
    MenuItemLine2: TMenuItem;
    MenuItemOnline: TMenuItem;
    MenuItemInvisible: TMenuItem;
    MenuItemIFA: TMenuItem;
    Memo2: TMemo;
    Label6: TLabel;
    EditRcvr: TEdit;
    ButtonSend: TButton;
    CheckBoxDelivery: TCheckBox;
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure ButtonLoginClick(Sender: TObject);
    procedure ButtonLogoffClick(Sender: TObject);
    procedure ButtonStatusClick(Sender: TObject);
    procedure MenuItemStatusClick(Sender: TObject);
    procedure ButtonSendClick(Sender: TObject);
  private
    FFuncsEvents: TFuncsEvents;
    FObimpProto: IObimpProto;
    FEventSink: IUnknown;
    FConnectionCookie: integer;
    FUniqSeq: DWord;
    function  GetUniqSequence: DWord;
  public
    procedure Log(s: string);
    procedure LoginBtnState(BtnEnabled: Boolean);

    //events funcs
    function  GetProtoIntfVer(): DWord;
    procedure ConnectionFailed();
    procedure LoginSuccess();
    procedure LoginFailed(LoginFailedCode: Word);
    procedure ServerBye(ByeReasonCode: Word);
    procedure MsgReceived(RcvdMsgInfo: TRcvdMsgInfo);
    procedure MsgAckRcvd(MsgAck: TMsgAck);
    procedure AuthReqRcvd(AuthReq: TRcvdAuthReq);
    procedure AuthReplyRcvd(AuthReply: TRcvdAuthReply);
    procedure AuthRevokeRcvd(AuthRevoke: TRcvdAuthRevoke);
  end;

var
  frmMain: TfrmMain;

implementation

{$R *.dfm}

{***************************************************************}
procedure TfrmMain.FormCreate(Sender: TObject);
begin
  ReportMemoryLeaksOnShutdown := True;

  //create events functions helper class
  FFuncsEvents := TFuncsEvents.Create;
  FFuncsEvents.GetProtoIntfVer  := GetProtoIntfVer;
  FFuncsEvents.ConnectionFailed := ConnectionFailed;
  FFuncsEvents.LoginSuccess     := LoginSuccess;
  FFuncsEvents.LoginFailed      := LoginFailed;
  FFuncsEvents.ServerBye        := ServerBye;
  FFuncsEvents.MsgReceived      := MsgReceived;
  FFuncsEvents.MsgAckRcvd       := MsgAckRcvd;
  FFuncsEvents.AuthReqRcvd      := AuthReqRcvd;
  FFuncsEvents.AuthReplyRcvd    := AuthReplyRcvd;
  FFuncsEvents.AuthRevokeRcvd   := AuthRevokeRcvd;

  //Creating OBIMP protocol com object
  FObimpProto := CreateComObject(CLASS_ObimpProto) as IObimpProto; // or CoObimpProto.Create;
  FEventSink  := TEventSink.Create(FFuncsEvents);

  //events
  InterfaceConnect(FObimpProto, IObimpProtoEvents, FEventSink, FConnectionCookie);

  //Interface version can be used for compatibility purposes
  Log('*** ObimpProto interface version = ' + IntToStr(FObimpProto.GetProtoIntfVer));

  //Set status selection popupmenu items tags
  MenuItemFFC.Tag       := PRES_STATUS_FREE_FOR_CHAT;
  MenuItemAtHome.Tag    := PRES_STATUS_AT_HOME;
  MenuItemAtWork.Tag    := PRES_STATUS_AT_WORK;
  MenuItemLunch.Tag     := PRES_STATUS_LUNCH;
  MenuItemAway.Tag      := PRES_STATUS_AWAY;
  MenuItemNA.Tag        := PRES_STATUS_NOT_AVAILABLE;
  MenuItemOccupied.Tag  := PRES_STATUS_OCCUPIED;
  MenuItemDND.Tag       := PRES_STATUS_DO_NOT_DISTURB;
  MenuItemOnline.Tag    := PRES_STATUS_ONLINE;
  MenuItemInvisible.Tag := PRES_STATUS_INVISIBLE;
  MenuItemIFA.Tag       := PRES_STATUS_INVISIBLE_FOR_ALL;

  ButtonStatus.Tag      := PRES_STATUS_ONLINE;
end;

{***************************************************************}
procedure TfrmMain.FormDestroy(Sender: TObject);
begin
  InterfaceDisconnect(FObimpProto, IObimpProtoEvents, FConnectionCookie);
  FObimpProto := nil;
  FEventSink  := nil;
  FFuncsEvents.Free;
end;

{***************************************************************}
function TfrmMain.GetUniqSequence: DWord;
begin
  //unique sequence needed for delivery message acknowledges
  Inc(FUniqSeq);
  Result := FUniqSeq;
end;

{***************************************************************}
procedure TfrmMain.Log(s: string);
begin
  Memo1.Lines.Add(TimeToStr(Now) + ' ' + s);
end;

{***************************************************************}
procedure TfrmMain.LoginBtnState(BtnEnabled: Boolean);
begin
  ButtonLogin.Enabled  := BtnEnabled;
  ButtonLogoff.Enabled := not BtnEnabled;
  ButtonSend.Enabled   := not BtnEnabled;

  EditAcc.Enabled      := BtnEnabled;
  EditPass.Enabled     := BtnEnabled;
  EditServer.Enabled   := BtnEnabled;
  EditPort.Enabled     := BtnEnabled;
end;

{***************************************************************}
procedure TfrmMain.ButtonLoginClick(Sender: TObject);
var LoginInfo: TLoginInfo;
    ClientInfo: TClientInfo;
    StatusInfo: TStatusInfo;
    iRes: integer;
begin
  //fill needed records for protocol interface
  with LoginInfo do
  begin
    Host     := Trim(EditServer.Text);            //must not be empty
    Port     := StrToIntDef(EditPort.Text, 7023); //must be > 0
    Account  := Trim(EditAcc.Text);               //must not be empty
    Password := EditPass.Text;                    //must not be empty
  end;//with

  with ClientInfo do
  begin
    ClientName     := 'ObimpClient'; //must not be empty
    ClientVerMaj   := 1;
    ClientVerMin   := 0;
    ClientVerRel   := 0;
    ClientVerBuild := 1;
    ClientType     := CLIENT_TYPE_BOT; //or CLIENT_TYPE_USER
    ClientLang     := LANGUAGE_CODE_ENGLISH; //GUI language of your application, must not be 0
  end;//with

  with StatusInfo do
  begin
    Status         := ButtonStatus.Tag;  //PRES_STATUS_ONLINE by default if not selected status
    StatusName     := ButtonStatus.Caption; //optional, can be empty
  end;//with

  //Login
  iRes := FObimpProto.Login(LoginInfo, ClientInfo, StatusInfo);
  if (iRes = FUNC_OPER_SUCCESS) then
  begin
    Log('* Connecting ...');
    LoginBtnState(False);
  end
  else
    ShowMessage('Can''t login. Some parameter is wrong.');
end;

{***************************************************************}
procedure TfrmMain.ButtonLogoffClick(Sender: TObject);
begin
  Log('* Logged off.');
  LoginBtnState(True);
  FObimpProto.Logoff;
end;

{***************************************************************}
procedure TfrmMain.ButtonStatusClick(Sender: TObject);
var P: TPoint;
begin
  if not GetCursorPos(P) then Exit;
  PopupMenuStatus.Popup(P.X, P.Y);
end;

{***************************************************************}
procedure TfrmMain.MenuItemStatusClick(Sender: TObject);
var StatusInfo: TStatusInfo;
    iRes: integer;
begin
  with StatusInfo do
  begin
    Status         := TMenuItem(Sender).Tag;
    StatusName     := TMenuItem(Sender).Caption; //optional, can be empty
  end;//with

  //change status
  iRes := FObimpProto.ChangeStatus(StatusInfo);
  if (iRes = FUNC_OPER_SUCCESS) then
  begin
    //remember selected status
    ButtonStatus.Caption := TMenuItem(Sender).Caption;
    ButtonStatus.Tag     := TMenuItem(Sender).Tag;
    Log('* Status changed');
  end
  else
    ShowMessage('Can''t change status. Some parameter is wrong.');
end;

{***************************************************************}
procedure TfrmMain.ButtonSendClick(Sender: TObject);
var SendMsgInfo: TSendMsgInfo;
    iRes: integer;
begin
  with SendMsgInfo do
  begin
    Account       := Trim(EditRcvr.Text);
    MsgUniqID     := GetUniqSequence;
    MsgText       := Trim(Memo2.Text);
    AckRequired   := CheckBoxDelivery.Checked;
    TranspID      := 0; //must be 0, this is for future server versions
  end;//with

  //send message
  iRes := FObimpProto.SendMsg(SendMsgInfo);
  if (iRes = FUNC_OPER_SUCCESS) then
  begin
    Memo2.Clear;
    if Memo2.CanFocus then Memo2.SetFocus;
    Log('* Message has been sent');
  end
  else
    ShowMessage('Can''t send message. Some parameter is wrong.');
end;

{***************************************************************}
function TfrmMain.GetProtoIntfVer: DWord;
begin
  //return SDK protocol interface version like it done below
  Result := PROTO_INTF_VERSION;
end;

{***************************************************************}
procedure TfrmMain.ConnectionFailed;
begin
  LoginBtnState(True);
  Log('# Event: Connection failed.');
end;

{***************************************************************}
procedure TfrmMain.LoginFailed(LoginFailedCode: Word);

  function LoginFailedDescr: string;
  begin
    Result := '';
    case LoginFailedCode of
      LOGIN_FAILED_INVALID_LOGIN    : Result := 'LOGIN_FAILED_INVALID_LOGIN';
      LOGIN_FAILED_TEMP_UNAVAILABLE : Result := 'LOGIN_FAILED_TEMP_UNAVAILABLE';
      LOGIN_FAILED_TOO_MANY_CLIENTS : Result := 'LOGIN_FAILED_TOO_MANY_CLIENTS';
      LOGIN_FAILED_OTHER_SRV_ERROR  : Result := 'LOGIN_FAILED_OTHER_SRV_ERROR';
    end;//case
  end;

begin
  LoginBtnState(True);
  Log('# Event: Login failed (code=' + LoginFailedDescr + ').');
end;

{***************************************************************}
procedure TfrmMain.LoginSuccess;
begin
  Log('# Event: Logged in.');
end;

{***************************************************************}
procedure TfrmMain.ServerBye(ByeReasonCode: Word);

  function ServerByeDescr: string;
  begin
    Result := '';
    case ByeReasonCode of
      BYE_REASON_SRV_SHUTDOWN       : Result := 'BYE_REASON_SRV_SHUTDOWN';
      BYE_REASON_CLI_NEW_LOGIN      : Result := 'BYE_REASON_CLI_NEW_LOGIN';
      BYE_REASON_ACCOUNT_KICKED     : Result := 'BYE_REASON_ACCOUNT_KICKED';
      BYE_REASON_INCORRECT_SEQ      : Result := 'BYE_REASON_INCORRECT_SEQ';
      BYE_REASON_INCORRECT_BEX_TYPE : Result := 'BYE_REASON_INCORRECT_BEX_TYPE';
      BYE_REASON_INCORRECT_BEX_SUB  : Result := 'BYE_REASON_INCORRECT_BEX_SUB';
      BYE_REASON_INCORRECT_BEX_STEP : Result := 'BYE_REASON_INCORRECT_BEX_STEP';
      BYE_REASON_TIMEOUT            : Result := 'BYE_REASON_TIMEOUT';
      BYE_REASON_INCORRECT_WTLD     : Result := 'BYE_REASON_INCORRECT_WTLD';
      BYE_REASON_NOT_ALLOWED        : Result := 'BYE_REASON_NOT_ALLOWED';
      BYE_REASON_FLOODING           : Result := 'BYE_REASON_FLOODING';
    end;//case
  end;

begin
  LoginBtnState(True);
  Log('# Event: ServerBye (code=' + ServerByeDescr + ').');
end;

{***************************************************************}
procedure TfrmMain.MsgReceived(RcvdMsgInfo: TRcvdMsgInfo);
var MsgAck: TMsgAck;
    iRes: integer;
begin
  Log('# Event: Message "' + RcvdMsgInfo.MsgText + '" received from "' + RcvdMsgInfo.Account + '"');

  //If sender requested message acknowledge then we should send it
  //(ofcourse if we are not in invisible statuses, act here like you wish).
  if RcvdMsgInfo.AckRequired then
  begin
    with MsgAck do
    begin
      Account   := RcvdMsgInfo.Account;
      MsgUniqID := RcvdMsgInfo.MsgUniqID;
      TranspID  := RcvdMsgInfo.TranspID;
    end;//with

    iRes := FObimpProto.SendMsgAck(MsgAck);
    if (iRes = FUNC_OPER_SUCCESS) then
      Log('* Delivery report to "' + RcvdMsgInfo.Account + '" has been sent')
    else
      Log('* Failed to send delivery report to "' + RcvdMsgInfo.Account + '"');
  end;
end;

{***************************************************************}
procedure TfrmMain.MsgAckRcvd(MsgAck: TMsgAck);
begin
  Log('# Event: Acknowledge received (delivered message to "' + MsgAck.Account + '")');
end;

{***************************************************************}
procedure TfrmMain.AuthReqRcvd(AuthReq: TRcvdAuthReq);
var AuthReply: TSendAuthReply;
    iRes: integer;
begin
  Log('# Event: Authorization request "' + AuthReq.ReasonText + '" received from "' + AuthReq.Account + '"');

  //grant or deny authorization request using:
  with AuthReply do
  begin
    Account    := AuthReq.Account;
    ReplyCode  := AUTH_REPLY_GRANTED; //or AUTH_REPLY_DENIED
    TranspID   := 0; //must be 0, this is for future server versions
  end;//with

  iRes := FObimpProto.SendAuthReply(AuthReply);
  if (iRes = FUNC_OPER_SUCCESS) then
    Log('* Authorization reply to "' + AuthReq.Account + '" has been sent')
  else
    Log('* Failed to send authorization reply to "' + AuthReq.Account + '"');
end;

{***************************************************************}
procedure TfrmMain.AuthReplyRcvd(AuthReply: TRcvdAuthReply);
var s: string;
begin
  s := '';
  case AuthReply.ReplyCode of
    AUTH_REPLY_GRANTED : s := 'granted';
    AUTH_REPLY_DENIED  : s := 'denied';
  end;//case

  Log('# Event: Authorization reply "' + s + '" received from "' + AuthReply.Account + '"');
end;

{***************************************************************}
procedure TfrmMain.AuthRevokeRcvd(AuthRevoke: TRcvdAuthRevoke);
begin
  Log('# Event: Authorization revoke "' + AuthRevoke.ReasonText + '" received from "' + AuthRevoke.Account + '"');
end;



end.
