unit obimp_ax_info;

interface

uses Windows, Variants, ObimpAX_TLB;

const
  //protocol interface version
  PROTO_INTF_VERSION = 1003;

  //protocol interface functions result values
  FUNC_OPER_SUCCESS = 0;
  FUNC_OPER_ERROR   = 1;

  //protocol login failed code values
  LOGIN_FAILED_INVALID_LOGIN     = 1;
  LOGIN_FAILED_TEMP_UNAVAILABLE  = 2;
  LOGIN_FAILED_TOO_MANY_CLIENTS  = 3;
  LOGIN_FAILED_OTHER_SRV_ERROR   = 4;

  //protocol bye reason codes
  //BYE_REASON_... can be found in u_obimp_const.pas

  procedure OutDebugStr(s: string);

  function  RecToVar_RcvdMsgInfo(const MsgInfo: TRcvdMsgInfo): Variant;
  function  RecToVar_MsgAck(const MsgAck: TMsgAck): Variant;
  function  RecToVar_RcvdAuthReq(const AuthReq: TRcvdAuthReq): Variant;
  function  RecToVar_RcvdAuthReply(const AuthReply: TRcvdAuthReply): Variant;
  function  RecToVar_RcvdAuthRevoke(const AuthRevoke: TRcvdAuthRevoke): Variant;

  function  VarToRec_RcvdMsgInfo(const Params: Variant): TRcvdMsgInfo;
  function  VarToRec_MsgAck(const Params: Variant): TMsgAck;
  function  VarToRec_RcvdAuthReq(const Params: Variant): TRcvdAuthReq;
  function  VarToRec_RcvdAuthReply(const Params: Variant): TRcvdAuthReply;
  function  VarToRec_RcvdAuthRevoke(const Params: Variant): TRcvdAuthRevoke;


type
  PArray_RcvdMsgInfo    = ^TArray_RcvdMsgInfo;
  TArray_RcvdMsgInfo    = array[0..9] of Variant;

  PArray_MsgAck         = ^TArray_MsgAck;
  TArray_MsgAck         = array[0..2] of Variant;

  PArray_RcvdAuthReq    = ^TArray_RcvdAuthReq;
  TArray_RcvdAuthReq    = array[0..4] of Variant;

  PArray_RcvdAuthReply  = ^TArray_RcvdAuthReply;
  TArray_RcvdAuthReply  = array[0..4] of Variant;

  PArray_RcvdAuthRevoke = ^TArray_RcvdAuthRevoke;
  TArray_RcvdAuthRevoke = array[0..4] of Variant;

implementation

procedure OutDebugStr(s: string);
begin
  OutputDebugString(PChar(s));
end;

function RecToVar_RcvdMsgInfo(const MsgInfo: TRcvdMsgInfo): Variant;
var rmi: PArray_RcvdMsgInfo;
begin
  Result := VarArrayCreate([0, 9], varVariant);

  rmi := VarArrayLock(Result);
  try
    rmi[0] := MsgInfo.Account;
    rmi[1] := MsgInfo.MsgUniqID;
    rmi[2] := MsgInfo.MsgType;
    rmi[3] := MsgInfo.MsgText;
    rmi[4] := MsgInfo.AckRequired;
    rmi[5] := MsgInfo.IsOfflineMsg;
    rmi[6] := MsgInfo.OfflineDT;
    rmi[7] := MsgInfo.IsSystemMsg;
    rmi[8] := MsgInfo.SysMsgPos;
    rmi[9] := MsgInfo.TranspID;
  finally
    VarArrayUnlock(Result);
  end;
end;

function RecToVar_MsgAck(const MsgAck: TMsgAck): Variant;
var ma: PArray_MsgAck;
begin
  Result := VarArrayCreate([0, 2], varVariant);

  ma := VarArrayLock(Result);
  try
    ma[0] := MsgAck.Account;
    ma[1] := MsgAck.MsgUniqID;
    ma[2] := MsgAck.TranspID;
  finally
    VarArrayUnlock(Result);
  end;
end;

function RecToVar_RcvdAuthReq(const AuthReq: TRcvdAuthReq): Variant;
var rar: PArray_RcvdAuthReq;
begin
  Result := VarArrayCreate([0, 4], varVariant);

  rar := VarArrayLock(Result);
  try
    rar[0] := AuthReq.Account;
    rar[1] := AuthReq.ReasonText;
    rar[2] := AuthReq.IsOfflineMsg;
    rar[3] := AuthReq.OfflineDT;
    rar[4] := AuthReq.TranspID;
  finally
    VarArrayUnlock(Result);
  end;
end;

function RecToVar_RcvdAuthReply(const AuthReply: TRcvdAuthReply): Variant;
var rar: PArray_RcvdAuthReply;
begin
  Result := VarArrayCreate([0, 4], varVariant);

  rar := VarArrayLock(Result);
  try
    rar[0] := AuthReply.Account;
    rar[1] := AuthReply.ReplyCode;
    rar[2] := AuthReply.IsOfflineMsg;
    rar[3] := AuthReply.OfflineDT;
    rar[4] := AuthReply.TranspID;
  finally
    VarArrayUnlock(Result);
  end;
end;

function RecToVar_RcvdAuthRevoke(const AuthRevoke: TRcvdAuthRevoke): Variant;
var rar: PArray_RcvdAuthRevoke;
begin
  Result := VarArrayCreate([0, 4], varVariant);

  rar := VarArrayLock(Result);
  try
    rar[0] := AuthRevoke.Account;
    rar[1] := AuthRevoke.ReasonText;
    rar[2] := AuthRevoke.IsOfflineMsg;
    rar[3] := AuthRevoke.OfflineDT;
    rar[4] := AuthRevoke.TranspID;
  finally
    VarArrayUnlock(Result);
  end;
end;

function VarToRec_RcvdMsgInfo(const Params: Variant): TRcvdMsgInfo;
var rmi: PArray_RcvdMsgInfo;
begin
  ZeroMemory(@Result, SizeOf(TRcvdMsgInfo));

  if (VarArrayLowBound(Params, 1) = 0) and (VarArrayHighBound(Params, 1) = 9) then
  begin
    rmi := VarArrayLock(Params);
    try
      Result.Account      := rmi[0];
      Result.MsgUniqID    := rmi[1];
      Result.MsgType      := rmi[2];
      Result.MsgText      := rmi[3];
      Result.AckRequired  := rmi[4];
      Result.IsOfflineMsg := rmi[5];
      Result.OfflineDT    := rmi[6];
      Result.IsSystemMsg  := rmi[7];
      Result.SysMsgPos    := rmi[8];
      Result.TranspID     := rmi[9];
    finally
      VarArrayUnlock(Params);
    end;
  end;
end;

function VarToRec_MsgAck(const Params: Variant): TMsgAck;
var ma: PArray_MsgAck;
begin
  ZeroMemory(@Result, SizeOf(TMsgAck));

  if (VarArrayLowBound(Params, 1) = 0) and (VarArrayHighBound(Params, 1) = 2) then
  begin
    ma := VarArrayLock(Params);
    try
      Result.Account   := ma[0];
      Result.MsgUniqID := ma[1];
      Result.TranspID  := ma[2];
    finally
      VarArrayUnlock(Params);
    end;
  end;
end;

function VarToRec_RcvdAuthReq(const Params: Variant): TRcvdAuthReq;
var rar: PArray_RcvdAuthReq;
begin
  ZeroMemory(@Result, SizeOf(TRcvdAuthReq));

  if (VarArrayLowBound(Params, 1) = 0) and (VarArrayHighBound(Params, 1) = 4) then
  begin
    rar := VarArrayLock(Params);
    try
      Result.Account      := rar[0];
      Result.ReasonText   := rar[1];
      Result.IsOfflineMsg := rar[2];
      Result.OfflineDT    := rar[3];
      Result.TranspID     := rar[4];
    finally
      VarArrayUnlock(Params);
    end;
  end;
end;

function VarToRec_RcvdAuthReply(const Params: Variant): TRcvdAuthReply;
var rar: PArray_RcvdAuthReply;
begin
  ZeroMemory(@Result, SizeOf(TRcvdAuthReply));

  if (VarArrayLowBound(Params, 1) = 0) and (VarArrayHighBound(Params, 1) = 4) then
  begin
    rar := VarArrayLock(Params);
    try
      Result.Account      := rar[0];
      Result.ReplyCode    := rar[1];
      Result.IsOfflineMsg := rar[2];
      Result.OfflineDT    := rar[3];
      Result.TranspID     := rar[4];
    finally
      VarArrayUnlock(Params);
    end;
  end;
end;

function VarToRec_RcvdAuthRevoke(const Params: Variant): TRcvdAuthRevoke;
var rar: PArray_RcvdAuthRevoke;
begin
  ZeroMemory(@Result, SizeOf(TRcvdAuthRevoke));

  if (VarArrayLowBound(Params, 1) = 0) and (VarArrayHighBound(Params, 1) = 4) then
  begin
    rar := VarArrayLock(Params);
    try
      Result.Account      := rar[0];
      Result.ReasonText   := rar[1];
      Result.IsOfflineMsg := rar[2];
      Result.OfflineDT    := rar[3];
      Result.TranspID     := rar[4];
    finally
      VarArrayUnlock(Params);
    end;
  end;
end;

end.
