unit u_events;

{$WARN SYMBOL_PLATFORM OFF}

interface

uses Windows, ActiveX, ObimpAX_TLB, SysUtils, obimp_ax_info;

type
  TOnAddLogEvent = procedure (LogStr: string) of object;

  TFunc_GetProtoIntfVer  = reference to function(): SYSUINT;
  TFunc_ConnectionFailed = reference to procedure();
  TFunc_LoginSuccess     = reference to procedure();
  TFunc_LoginFailed      = reference to procedure(LoginFailedCode: Word);
  TFunc_ServerBye        = reference to procedure(ByeReasonCode: Word);
  TFunc_MsgReceived      = reference to procedure(RcvdMsgInfo: TRcvdMsgInfo);
  TFunc_MsgAckRcvd       = reference to procedure(MsgAck: TMsgAck);
  TFunc_AuthReqRcvd      = reference to procedure(AuthReq: TRcvdAuthReq);
  TFunc_AuthReplyRcvd    = reference to procedure(AuthReply: TRcvdAuthReply);
  TFunc_AuthRevokeRcvd   = reference to procedure(AuthRevoke: TRcvdAuthRevoke);

  TFuncsEvents = class
  public
    GetProtoIntfVer   : TFunc_GetProtoIntfVer;
    ConnectionFailed  : TFunc_ConnectionFailed;
    LoginSuccess      : TFunc_LoginSuccess;
    LoginFailed       : TFunc_LoginFailed;
    ServerBye         : TFunc_ServerBye;
    MsgReceived       : TFunc_MsgReceived;
    MsgAckRcvd        : TFunc_MsgAckRcvd;
    AuthReqRcvd       : TFunc_AuthReqRcvd;
    AuthReplyRcvd     : TFunc_AuthReplyRcvd;
    AuthRevokeRcvd    : TFunc_AuthRevokeRcvd;
  end;

  TEventSink = class(TInterfacedObject, IUnknown, IDispatch)
  private
    FFuncsEvents: TFuncsEvents;
    function QueryInterface(const IID: TGUID; out Obj): HResult; stdcall;
    function GetTypeInfoCount(out Count: Integer): HResult; stdcall;
    function GetTypeInfo(Index, LocaleID: Integer; out TypeInfo): HResult; stdcall;
    function GetIDsOfNames(const IID: TGUID; Names: Pointer; NameCount, LocaleID: Integer; DispIDs: Pointer): HResult; stdcall;
    function Invoke(DispID: Integer; const IID: TGUID; LocaleID: Integer; Flags: Word; var Params; VarResult, ExcepInfo, ArgErr: Pointer): HResult; stdcall;
  public
    constructor Create(FuncsEvents: TFuncsEvents);
  end;

implementation

{ TEventSink }
procedure OutDebugStr(s: string);
begin
  OutputDebugString(PChar(s));
end;

constructor TEventSink.Create(FuncsEvents: TFuncsEvents);
begin
  inherited Create;
  FFuncsEvents := FuncsEvents;
end;

function TEventSink.QueryInterface(const IID: TGUID; out Obj): HResult;
begin
  if GetInterFace(IID,Obj) then
    Result := S_OK
  else
  if IsEqualIID(IID, IObimpProtoEvents) then
    Result := QueryInterface(IDispatch, Obj)
  else
    Result := E_NOINTERFACE;
end;

function TEventSink.GetIDsOfNames(const IID: TGUID; Names: Pointer; NameCount, LocaleID: Integer; DispIDs: Pointer): HResult;
begin
  Result := S_OK;
end;

function TEventSink.GetTypeInfo(Index, LocaleID: Integer; out TypeInfo): HResult;
begin
  Result := S_OK;
end;

function TEventSink.GetTypeInfoCount(out Count: Integer): HResult;
begin
  Result := S_OK;
end;

function TEventSink.Invoke(DispID: Integer; const IID: TGUID; LocaleID: Integer; Flags: Word; var Params; VarResult, ExcepInfo, ArgErr: Pointer): HResult;
var V: ^Variant absolute VarResult;
    P: ^Variant absolute Params;
begin
  Result := S_OK;
  if not Assigned(FFuncsEvents) then Exit;

  case DispID of
    201: V^ := FFuncsEvents.GetProtoIntfVer();
    202: FFuncsEvents.ConnectionFailed();
    203: FFuncsEvents.LoginSuccess();
    204: FFuncsEvents.LoginFailed(OleVariant(TDispParams(Params).rgvarg^[0]));
    205: FFuncsEvents.ServerBye(OleVariant(TDispParams(Params).rgvarg^[0]));
    206: FFuncsEvents.MsgReceived(VarToRec_RcvdMsgInfo(P^));
    207: FFuncsEvents.MsgAckRcvd(VarToRec_MsgAck(P^));
    208: FFuncsEvents.AuthReqRcvd(VarToRec_RcvdAuthReq(P^));
    209: FFuncsEvents.AuthReplyRcvd(VarToRec_RcvdAuthReply(P^));
    210: FFuncsEvents.AuthRevokeRcvd(VarToRec_RcvdAuthRevoke(P^));
  end;
end;


end.
