unit f_main;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, Menus, obimp_dll, u_obimp_const, u_obimp_codes;

type
  TObimpEvents = class(TInterfacedObject, IObimpEvents)
  public
    {==== Interface functions ====}
    function  GetProtoIntfVer(): DWord; stdcall;
    procedure ConnectionFailed(); stdcall;
    procedure LoginSuccess(); stdcall;
    procedure LoginFailed(LoginFailedCode: Word); stdcall;
    procedure ServerBye(ByeReasonCode: Word); stdcall;
    procedure MsgReceived(RcvdMsgInfo: pRcvdMsgInfo); stdcall;
    procedure MsgAckRcvd(MsgAck: pMsgAck); stdcall;
    procedure AuthReqRcvd(AuthReq: pRcvdAuthReq); stdcall;
    procedure AuthReplyRcvd(AuthReply: pRcvdAuthReply); stdcall;
    procedure AuthRevokeRcvd(AuthRevoke: pRcvdAuthRevoke); stdcall;
  end;

  TfrmMain = class(TForm)
    EditAcc: TEdit;
    EditPass: TEdit;
    Label1: TLabel;
    Label2: TLabel;
    ButtonLogin: TButton;
    ButtonLogoff: TButton;
    Memo1: TMemo;
    EditServer: TEdit;
    Label3: TLabel;
    EditPort: TEdit;
    Label4: TLabel;
    ButtonStatus: TButton;
    Label5: TLabel;
    PopupMenuStatus: TPopupMenu;
    MenuItemFFC: TMenuItem;
    MenuItemAtHome: TMenuItem;
    MenuItemAtWork: TMenuItem;
    MenuItemLine1: TMenuItem;
    MenuItemLunch: TMenuItem;
    MenuItemAway: TMenuItem;
    MenuItemNA: TMenuItem;
    MenuItemOccupied: TMenuItem;
    MenuItemDND: TMenuItem;
    MenuItemLine2: TMenuItem;
    MenuItemOnline: TMenuItem;
    MenuItemInvisible: TMenuItem;
    MenuItemIFA: TMenuItem;
    Memo2: TMemo;
    Label6: TLabel;
    EditRcvr: TEdit;
    ButtonSend: TButton;
    CheckBoxDelivery: TCheckBox;
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure ButtonLoginClick(Sender: TObject);
    procedure ButtonLogoffClick(Sender: TObject);
    procedure ButtonStatusClick(Sender: TObject);
    procedure MenuItemStatusClick(Sender: TObject);
    procedure ButtonSendClick(Sender: TObject);
  private
    FLibHandle    : THandle;
    FUniqSeq      : DWord;
    FProtoEvents  : TObimpEvents;
    function  GetUniqSequence: DWord;
  public
    ObimpEvents : IObimpEvents;
    ObimpProto  : IObimpProto;
    procedure Log(s: string);
    procedure LoginBtnState(BtnEnabled: Boolean);
  end;

var
  frmMain: TfrmMain;

implementation

{$R *.dfm}

type
  TFuncCreateObimp = function(ObimpEvents: IObimpEvents): IObimpProto; stdcall;

{***************************************************************}
procedure TfrmMain.FormCreate(Sender: TObject);
var s: string;
    FuncCreateObimp: TFuncCreateObimp;
begin
  ReportMemoryLeaksOnShutdown := True;

  //Create protocol events class
  FProtoEvents := TObimpEvents.Create;

  //Increase interface references counter, else our events class will be
  //freed after destroying ObimpProto interface.
  ObimpEvents := FProtoEvents;

  //Dynamically loading micro OBIMP library...
  FLibHandle := SafeLoadLibrary(OBIMP_DLL_FILE_NAME);
  if (FLibHandle = 0) then
  begin
    s := '"' + OBIMP_DLL_FILE_NAME + '" file not found! Program will be closed.';
    MessageBox(Handle, PChar(s), 'Error', MB_ICONERROR or MB_OK);
    Halt;
  end
  else
    Log('*** Library "' + OBIMP_DLL_FILE_NAME + '" loaded successfully.');

  //Getting CreateObimp function address from OBIMP library...
  @FuncCreateObimp := GetProcAddress(FLibHandle, DLL_EXP_FNAME_CREATE_OBIMP);
  if (@FuncCreateObimp = nil) then
  begin
    s := 'Function "' + DLL_EXP_FNAME_CREATE_OBIMP + '" not found! Program will be closed.';
    MessageBox(Handle, PChar(s), 'Error', MB_ICONERROR or MB_OK);
    FreeLibrary(FLibHandle);
    Halt;
  end;

  //Creating OBIMP protocol interface instance
  ObimpProto := FuncCreateObimp(ObimpEvents);

  //Interface version can be used for compatibility purposes
  Log('*** ObimpProto interface version = ' + IntToStr(ObimpProto.GetProtoIntfVer));

  //Set status selection popupmenu items tags
  MenuItemFFC.Tag       := PRES_STATUS_FREE_FOR_CHAT;
  MenuItemAtHome.Tag    := PRES_STATUS_AT_HOME;
  MenuItemAtWork.Tag    := PRES_STATUS_AT_WORK;
  MenuItemLunch.Tag     := PRES_STATUS_LUNCH;
  MenuItemAway.Tag      := PRES_STATUS_AWAY;
  MenuItemNA.Tag        := PRES_STATUS_NOT_AVAILABLE;
  MenuItemOccupied.Tag  := PRES_STATUS_OCCUPIED;
  MenuItemDND.Tag       := PRES_STATUS_DO_NOT_DISTURB;
  MenuItemOnline.Tag    := PRES_STATUS_ONLINE;
  MenuItemInvisible.Tag := PRES_STATUS_INVISIBLE;
  MenuItemIFA.Tag       := PRES_STATUS_INVISIBLE_FOR_ALL;

  ButtonStatus.Tag      := PRES_STATUS_ONLINE;
end;

{***************************************************************}
procedure TfrmMain.FormDestroy(Sender: TObject);
begin
  //destroy protocol interface instance
  ObimpProto := nil;

  //nil events interface, it will free FProtoEvents class
  ObimpEvents := nil;

  if (FLibHandle > 0) then
    FreeLibrary(FLibHandle);
end;

{***************************************************************}
function TfrmMain.GetUniqSequence: DWord;
begin
  //unique sequence needed for delivery message acknowledges
  Inc(FUniqSeq);
  Result := FUniqSeq;
end;

{***************************************************************}
procedure TfrmMain.Log(s: string);
begin
  Memo1.Lines.Add(TimeToStr(Now) + ' ' + s);
end;

{***************************************************************}
procedure TfrmMain.LoginBtnState(BtnEnabled: Boolean);
begin
  ButtonLogin.Enabled  := BtnEnabled;
  ButtonLogoff.Enabled := not BtnEnabled;
  ButtonSend.Enabled   := not BtnEnabled;

  EditAcc.Enabled      := BtnEnabled;
  EditPass.Enabled     := BtnEnabled;
  EditServer.Enabled   := BtnEnabled;
  EditPort.Enabled     := BtnEnabled;
end;

{***************************************************************}
procedure TfrmMain.ButtonLoginClick(Sender: TObject);
var LoginInfo: TLoginInfo;
    ClientInfo: TClientInfo;
    StatusInfo: TStatusInfo;
    iRes: integer;
begin
  //fill needed records for protocol interface
  with LoginInfo do
  begin
    Host     := Trim(EditServer.Text);            //must not be empty
    Port     := StrToIntDef(EditPort.Text, 7023); //must be > 0
    Account  := Trim(EditAcc.Text);               //must not be empty
    Password := EditPass.Text;                    //must not be empty
  end;//with

  with ClientInfo do
  begin
    ClientName     := 'ObimpClient'; //must not be empty
    ClientVerMaj   := 1;
    ClientVerMin   := 0;
    ClientVerRel   := 0;
    ClientVerBuild := 1;
    ClientType     := CLIENT_TYPE_BOT; //or CLIENT_TYPE_USER
    ClientLang     := LANGUAGE_CODE_ENGLISH; //GUI language of your application, must not be 0
  end;//with

  with StatusInfo do
  begin
    Status         := ButtonStatus.Tag;  //PRES_STATUS_ONLINE by default if not selected status
    StatusName     := ButtonStatus.Caption; //optional, can be empty
  end;//with

  //Login
  iRes := ObimpProto.Login(@LoginInfo, @ClientInfo, @StatusInfo);
  if (iRes = FUNC_OPER_SUCCESS) then
  begin
    Log('* Connecting ...');
    LoginBtnState(False);
  end
  else
    ShowMessage('Can''t login. Some parameter is wrong.');
end;

{***************************************************************}
procedure TfrmMain.ButtonLogoffClick(Sender: TObject);
begin
  Log('* Logged off.');
  LoginBtnState(True);
  ObimpProto.Logoff;
end;

{***************************************************************}
procedure TfrmMain.ButtonStatusClick(Sender: TObject);
var P: TPoint;
begin
  if not GetCursorPos(P) then Exit;
  PopupMenuStatus.Popup(P.X, P.Y);
end;

{***************************************************************}
procedure TfrmMain.MenuItemStatusClick(Sender: TObject);
var StatusInfo: TStatusInfo;
    iRes: integer;
begin
  with StatusInfo do
  begin
    Status         := TMenuItem(Sender).Tag;
    StatusName     := TMenuItem(Sender).Caption; //optional, can be empty
  end;//with

  //change status
  iRes := ObimpProto.ChangeStatus(@StatusInfo);
  if (iRes = FUNC_OPER_SUCCESS) then
  begin
    //remember selected status
    ButtonStatus.Caption := TMenuItem(Sender).Caption;
    ButtonStatus.Tag     := TMenuItem(Sender).Tag;
    Log('* Status changed');
  end
  else
    ShowMessage('Can''t change status. Some parameter is wrong.');
end;

{***************************************************************}
procedure TfrmMain.ButtonSendClick(Sender: TObject);
var SendMsgInfo: TSendMsgInfo;
    iRes: integer;
begin
  with SendMsgInfo do
  begin
    Account       := Trim(EditRcvr.Text);
    MsgUniqID     := GetUniqSequence;
    MsgText       := Trim(Memo2.Text);
    AckRequired   := CheckBoxDelivery.Checked;
    TranspID      := 0; //must be 0, this is for future server versions
  end;//with

  //send message
  iRes := ObimpProto.SendMsg(@SendMsgInfo);
  if (iRes = FUNC_OPER_SUCCESS) then
  begin
    Memo2.Clear;
    if Memo2.CanFocus then Memo2.SetFocus;
    Log('* Message has been sent');
  end
  else
    ShowMessage('Can''t send message. Some parameter is wrong.');
end;


{ TObimpEvents }
{***************************************************************}
function TObimpEvents.GetProtoIntfVer: DWord;
begin
  //return SDK protocol interface version like it done below
  Result := PROTO_INTF_VERSION;
end;

{***************************************************************}
procedure TObimpEvents.ConnectionFailed;
begin
  frmMain.LoginBtnState(True);
  frmMain.Log('# Event: Connection failed.');
end;

{***************************************************************}
procedure TObimpEvents.LoginFailed(LoginFailedCode: Word);

  function LoginFailedDescr: string;
  begin
    Result := '';
    case LoginFailedCode of
      LOGIN_FAILED_INVALID_LOGIN    : Result := 'LOGIN_FAILED_INVALID_LOGIN';
      LOGIN_FAILED_TEMP_UNAVAILABLE : Result := 'LOGIN_FAILED_TEMP_UNAVAILABLE';
      LOGIN_FAILED_TOO_MANY_CLIENTS : Result := 'LOGIN_FAILED_TOO_MANY_CLIENTS';
      LOGIN_FAILED_OTHER_SRV_ERROR  : Result := 'LOGIN_FAILED_OTHER_SRV_ERROR';
    end;//case
  end;

begin
  frmMain.LoginBtnState(True);
  frmMain.Log('# Event: Login failed (code=' + LoginFailedDescr + ').');
end;

{***************************************************************}
procedure TObimpEvents.LoginSuccess;
begin
  frmMain.Log('# Event: Logged in.');
end;

{***************************************************************}
procedure TObimpEvents.ServerBye(ByeReasonCode: Word);

  function ServerByeDescr: string;
  begin
    Result := '';
    case ByeReasonCode of
      BYE_REASON_SRV_SHUTDOWN       : Result := 'BYE_REASON_SRV_SHUTDOWN';
      BYE_REASON_CLI_NEW_LOGIN      : Result := 'BYE_REASON_CLI_NEW_LOGIN';
      BYE_REASON_ACCOUNT_KICKED     : Result := 'BYE_REASON_ACCOUNT_KICKED';
      BYE_REASON_INCORRECT_SEQ      : Result := 'BYE_REASON_INCORRECT_SEQ';
      BYE_REASON_INCORRECT_BEX_TYPE : Result := 'BYE_REASON_INCORRECT_BEX_TYPE';
      BYE_REASON_INCORRECT_BEX_SUB  : Result := 'BYE_REASON_INCORRECT_BEX_SUB';
      BYE_REASON_INCORRECT_BEX_STEP : Result := 'BYE_REASON_INCORRECT_BEX_STEP';
      BYE_REASON_TIMEOUT            : Result := 'BYE_REASON_TIMEOUT';
      BYE_REASON_INCORRECT_WTLD     : Result := 'BYE_REASON_INCORRECT_WTLD';
      BYE_REASON_NOT_ALLOWED        : Result := 'BYE_REASON_NOT_ALLOWED';
      BYE_REASON_FLOODING           : Result := 'BYE_REASON_FLOODING';
    end;//case
  end;

begin
  frmMain.LoginBtnState(True);
  frmMain.Log('# Event: ServerBye (code=' + ServerByeDescr + ').');
end;

{***************************************************************}
procedure TObimpEvents.MsgReceived(RcvdMsgInfo: pRcvdMsgInfo);
var MsgAck: TMsgAck;
    iRes: integer;
begin
  frmMain.Log('# Event: Message "' + RcvdMsgInfo^.MsgText + '" received from "' + RcvdMsgInfo^.Account + '"');

  //If sender requested message acknowledge then we should send it
  //(ofcourse if we are not in invisible statuses, act here like you wish).
  if RcvdMsgInfo^.AckRequired then
  begin
    with MsgAck do
    begin
      Account   := RcvdMsgInfo^.Account;
      MsgUniqID := RcvdMsgInfo^.MsgUniqID;
      TranspID  := RcvdMsgInfo^.TranspID;
    end;//with

    iRes := frmMain.ObimpProto.SendMsgAck(@MsgAck);
    if (iRes = FUNC_OPER_SUCCESS) then
      frmMain.Log('* Delivery report to "' + RcvdMsgInfo^.Account + '" has been sent')
    else
      frmMain.Log('* Failed to send delivery report to "' + RcvdMsgInfo^.Account + '"');
  end;
end;

{***************************************************************}
procedure TObimpEvents.MsgAckRcvd(MsgAck: pMsgAck);
begin
  frmMain.Log('# Event: Acknowledge received (delivered message to "' + MsgAck^.Account + '")');
end;

{***************************************************************}
procedure TObimpEvents.AuthReqRcvd(AuthReq: pRcvdAuthReq);
var AuthReply: TSendAuthReply;
    iRes: integer;
begin
  frmMain.Log('# Event: Authorization request "' + AuthReq^.ReasonText + '" received from "' + AuthReq^.Account + '"');

  //grant or deny authorization request using:
  with AuthReply do
  begin
    Account    := AuthReq^.Account;
    ReplyCode  := AUTH_REPLY_GRANTED; //or AUTH_REPLY_DENIED
    TranspID   := 0; //must be 0, this is for future server versions
  end;//with

  iRes := frmMain.ObimpProto.SendAuthReply(@AuthReply);
  if (iRes = FUNC_OPER_SUCCESS) then
    frmMain.Log('* Authorization reply to "' + AuthReq^.Account + '" has been sent')
  else
    frmMain.Log('* Failed to send authorization reply to "' + AuthReq^.Account + '"');
end;

{***************************************************************}
procedure TObimpEvents.AuthReplyRcvd(AuthReply: pRcvdAuthReply);
var s: string;
begin
  s := '';
  case AuthReply^.ReplyCode of
    AUTH_REPLY_GRANTED : s := 'granted';
    AUTH_REPLY_DENIED  : s := 'denied';
  end;//case

  frmMain.Log('# Event: Authorization reply "' + s + '" received from "' + AuthReply^.Account + '"');
end;

{***************************************************************}
procedure TObimpEvents.AuthRevokeRcvd(AuthRevoke: pRcvdAuthRevoke);
begin
  frmMain.Log('# Event: Authorization revoke "' + AuthRevoke^.ReasonText + '" received from "' + AuthRevoke^.Account + '"');
end;



end.
